package eus.urko.appresidencia.android.Activity;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Point;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;

import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.ScrollView;
import android.widget.TextView;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;

import eus.urko.appresidencia.android.R;

/**
 * Actividad para identificar al usuario
 */
public class LoginActivity extends AppCompatActivity {
    // TAG para identificar la Activity
    private static final String TAG = "LoginActivity";

    // Referencias UI
    private EditText etEmail;
    private EditText etContraseña;
    private TextView tvOlvidadoContraseña;
    private TextView tvRegistrarse;
    private Button btIndetificarse;
    private ProgressBar pbLogin;
    private ScrollView svIdentificacion;
    private ImageView ivResa;

    // Objetos de Firebase
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        // Cargar referencias UI
        etEmail = (EditText) findViewById(R.id.etEmail);
        etContraseña = (EditText) findViewById(R.id.etContraseña);
        tvOlvidadoContraseña = (TextView) findViewById(R.id.tvOlvidadoContraseña);
        tvRegistrarse = (TextView) findViewById(R.id.tvRegistrarse);
        btIndetificarse = (Button) findViewById(R.id.btIndetificarse);
        svIdentificacion = (ScrollView) findViewById(R.id.svIdentificacion);
        pbLogin = (ProgressBar) findViewById(R.id.pbLogin);
        ivResa = (ImageView) findViewById(R.id.ivResa);

        //  Firebase
        mAuth = FirebaseAuth.getInstance();
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                // Listener que controla si el usuario se ha identificado
                FirebaseUser user = firebaseAuth.getCurrentUser();
                if (user != null) {
                    // Identificado, pasa a la actividad principal
                    startActivity(new Intent(getApplication(), MainActivity.class));
                    finish();
                }
            }
        };

        // Listener que se activa al pulsar el boton de identificacion
        btIndetificarse.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                // Proceso de identificacion
                intentarIdentificarse();
            }
        });

        // Listener que se activa al pulsar las teclas correspondientes del teclado
        etContraseña.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == R.id.btIndetificarse || id == EditorInfo.IME_NULL) {
                    intentarIdentificarse();
                    return true;
                }
                return false;
            }
        });

        // Listener que se activa al pinchar sobre tvRegistarse
        tvRegistrarse.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                intentarRegistrarse();
            }
        });

        // Listener que se activa al pinchar sobre tvOlvidadoContraseña
        tvOlvidadoContraseña.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                intentarRecuperarContraseña();
            }
        });
    }

    @Override
    public void onStart() {
        super.onStart();
        // Añadir listener de identificacion
        mAuth.addAuthStateListener(mAuthListener);
        cerrarTeclado();
    }

    @Override
    public void onStop() {
        super.onStop();
        if (mAuthListener != null)
            // Quitar listener de identificacion
            mAuth.removeAuthStateListener(mAuthListener);

    }

    // Comprobar si el email es valido
    private boolean isValidoEmail(String email) {
        return email.contains("@");
    }

    // Comprobar si la contraseña es valida
    private boolean isValidaContraseña(String contraseña) {
        return contraseña.length() > 4;
    }

    /**
     * Intenta identificar al usuario con los datos introducidos.
     * En caso de haber algun error, se muestra en pantalla
     */
    private void intentarIdentificarse() {
        cerrarTeclado();
        // Quitar errores
        etEmail.setError(null);
        etContraseña.setError(null);

        // Obtener el email y contraseña introducido
        String email = etEmail.getText().toString(); // Obtener el email
        String contraseña = etContraseña.getText().toString(); // Obtener la contraseña
        boolean cancelar = false; // Permite saber si hay que cancelar el intento de identificacion
        View focusView = null; // View donde hacer el focus en caso de error

        // Comprobar que la contraseña es valida
        if (TextUtils.isEmpty(contraseña) || !isValidaContraseña(contraseña)) {
            etContraseña.setError(getString(R.string.contraseña_inconrrecta));
            focusView = etContraseña;
            cancelar = true;
        }

        // Comprobar que el email es correcto
        if (TextUtils.isEmpty(email)) {
            etEmail.setError(getString(R.string.campo_requerido));
            focusView = etEmail;
            cancelar = true;
        } else if (!isValidoEmail(email)) {
            etEmail.setError(getString(R.string.email_incorrecto));
            focusView = etEmail;
            cancelar = true;
        }

        if (cancelar)
            // Ha habido un error, el proceso se para
            // Se hace focus en el error
            focusView.requestFocus();
         else {
            // Mostrar el proceso de identificacion (spinner)
            showProgress(true);
            mAuth.signInWithEmailAndPassword(email, contraseña)
                    .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            // Quitar el proceso de identificacion (spinner)
                            showProgress(false);

                            // Si la identificación es correcta, el propio listener de Firebase pasará a la siguiente activity
                            if (!task.isSuccessful()) {
                                // En caso de que haya ido mal, mostrar error
                                Snackbar.make(svIdentificacion, R.string.fallo_identificacion, Snackbar.LENGTH_SHORT).show();
                                etContraseña.setError(getString(R.string.contraseña_inconrrecta));
                                etContraseña.requestFocus();
                            }
                        }
                    });
        }
    }

    /**
     * Intenta mandar el email de recuperacion al usuario con los datos introducidos.
     * En caso de haber algun error, se muestra en pantalla
     */
    private void intentarRecuperarContraseña() {
        cerrarTeclado();
        // Quitar errores
        etEmail.setError(null);
        etContraseña.setError(null);

        // Obtener el email introducido
        String email = etEmail.getText().toString(); // Obtener el email
        boolean cancelar = false; // Permite saber si hay que cancelar el intento de identificacion
        View focusView = null; // View donde hacer el focus en caso de error

        // Comprobar que el email es correcto
        if (TextUtils.isEmpty(email)) {
            etEmail.setError(getString(R.string.campo_requerido));
            focusView = etEmail;
            cancelar = true;
        } else if (!isValidoEmail(email)) {
            etEmail.setError(getString(R.string.email_incorrecto));
            focusView = etEmail;
            cancelar = true;
        }

        if (cancelar){
            // Ha habido un error, el proceso se para
            // Se hace focus en el error
            focusView.requestFocus();

            final AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setMessage(R.string.dialogo_recuperar_contraseña_mensaje)
                    .setTitle(R.string.dialogo_recuperar_contraseña_titulo);
            builder.setPositiveButton(R.string.aceptar_dialogo, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    dialog.cancel();
                }
            });
            AlertDialog dialog = builder.create();
            dialog.show();
        }


        else {
            // Mostrar el proceso de identificacion (spinner)
            showProgress(true);
            mAuth.sendPasswordResetEmail(email)
                    .addOnCompleteListener(new OnCompleteListener<Void>() {
                        @Override
                        public void onComplete(@NonNull Task<Void> task) {
                            // Quitar el proceso de identificacion (spinner)
                            showProgress(false);
                            if (task.isSuccessful())
                                Snackbar.make(svIdentificacion, R.string.email_enviado_ok, Snackbar.LENGTH_SHORT).show();
                            else
                                Snackbar.make(svIdentificacion, getString(R.string.email_enviado_error) + task.getException().getMessage(), Snackbar.LENGTH_SHORT).show();


                        }
                    });
        }
    }

    // Función que intenta registrar a un usuario
    // Se abre ventana con informacion
    private void intentarRegistrarse() {
        cerrarTeclado();
        final AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.mensaje_crear_usuario)
                .setTitle(R.string.titulo_crear_usuario);
        builder.setPositiveButton(R.string.aceptar_dialogo, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog dialog = builder.create();
        dialog.show();
    }



    /**
     * Muestra o oculta el proceso de identificacion
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
    private void showProgress(final boolean show) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
            // Hacer animacion de la UI
            int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);

            svIdentificacion.setVisibility(show ? View.GONE : View.VISIBLE);
            svIdentificacion.animate().setDuration(shortAnimTime).alpha(
                    show ? 0 : 1).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    svIdentificacion.setVisibility(show ? View.GONE : View.VISIBLE);
                }
            });

            pbLogin.setVisibility(show ? View.VISIBLE : View.GONE);
            pbLogin.animate().setDuration(shortAnimTime).alpha(
                    show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    pbLogin.setVisibility(show ? View.VISIBLE : View.GONE);
                }
            });
        } else {
            // No esta disponible las API, asi que mostrar o ocular las partes más importantes de la UI
            pbLogin.setVisibility(show ? View.VISIBLE : View.GONE);
            svIdentificacion.setVisibility(show ? View.GONE : View.VISIBLE);
        }
    }

    // Cierra el teclado virtual, siempre y cuando sea posible
    private void cerrarTeclado(){
        View view = this.getCurrentFocus();
        if (view != null) {
            InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

}

