package eus.urko.appresidencia.android.Activity;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.NavigationView;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.widget.TextView;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import eus.urko.appresidencia.android.Dominio.Usuario;
import eus.urko.appresidencia.android.Dominio.Valoracion;
import eus.urko.appresidencia.android.Fragments.AjustesFragment;
import eus.urko.appresidencia.android.Fragments.InformacionFragment;
import eus.urko.appresidencia.android.Fragments.MainActivityFragment;
import eus.urko.appresidencia.android.Fragments.MonederoFragment;
import eus.urko.appresidencia.android.Fragments.OfertasFragment;
import eus.urko.appresidencia.android.Fragments.PeticionesFragment;
import eus.urko.appresidencia.android.Fragments.ValorarFragment;
import eus.urko.appresidencia.android.Fragments.VerMenuPorDiaFragment;
import eus.urko.appresidencia.android.R;

public class MainActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener,
        VerMenuPorDiaFragment.OnFragmentInteractionListener,
        OfertasFragment.OnFragmentInteractionListener,
        AjustesFragment.OnFragmentInteractionListener,
        MonederoFragment.OnFragmentInteractionListener,
        InformacionFragment.OnFragmentInteractionListener,
        PeticionesFragment.OnFragmentInteractionListener,
        ValorarFragment.OnFragmentInteractionListener{
    // TAG para identificar la Activity
    private static final String TAG = "MainActivity";

    // Objetos de Firebase
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;

    // Referencias UI
    private Toolbar toolbarMain;
    private DrawerLayout dlMain;
    private NavigationView nvMain;
    private ActionBarDrawerToggle actionBarDrawerToggle;
    private TextView tvNombreUsuario;
    private TextView tvEmailUsuario;

    // Objetos estaticos sobre el usuario identificado
    public static FirebaseUser firebaseUser = null;
    public static Usuario usuario = null;

    // Objetos estaticos de referencia de la base de datos
    public static FirebaseDatabase refRoot;
    public static DatabaseReference refUsuarios;
    public static DatabaseReference refUsuarioActual;
    public static DatabaseReference refResindecias;
    public static DatabaseReference refCursos;
    public static DatabaseReference refDias;
    public static DatabaseReference refDesayunos;
    public static DatabaseReference refComidas;
    public static DatabaseReference refAlergenos;
    public static DatabaseReference refOfertas;
    public static DatabaseReference refTransacciones;
    public static DatabaseReference refTakeaway;
    public static DatabaseReference refValoraciones;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Cargar referencias UI
        toolbarMain = (Toolbar) findViewById(R.id.toolbarMain);
        dlMain = (DrawerLayout) findViewById(R.id.dlMain);
        nvMain = (NavigationView) findViewById(R.id.nvMain);
        tvNombreUsuario = (TextView) nvMain.getHeaderView(0).findViewById(R.id.tvNombreUsuario);
        tvEmailUsuario = (TextView) nvMain.getHeaderView(0).findViewById(R.id.tvEmailUsuario);

        // Cargar toolbar
        setSupportActionBar(toolbarMain);

        // Cargar el menú de navegación
        actionBarDrawerToggle = new ActionBarDrawerToggle(
                this, dlMain, toolbarMain, R.string.menu_navegacion_abierto, R.string.menu_navegacion_cerrado);
        dlMain.setDrawerListener(actionBarDrawerToggle);
        actionBarDrawerToggle.syncState();

        // Firebase Login
        mAuth = FirebaseAuth.getInstance();

        // Listener que comprueba en todo momento si el usuario está identificado
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                FirebaseUser user = firebaseAuth.getCurrentUser();
                firebaseUser = user;
                if (user == null) {
                    Intent intent = new Intent(getApplication(), LoginActivity.class);
                    startActivity(intent);
                    finish();
                } else {
                    refUsuarioActual = refUsuarios.child(user.getUid());
                    tvEmailUsuario.setText(user.getEmail());
                    inicializarListenerUsuario();
                }
            }
        };

        // Cargar referencias de la base de datos
        refRoot  = FirebaseDatabase.getInstance();
        refUsuarios = refRoot.getReference("usuarios");
        refResindecias = refRoot.getReference("residencias");
        refCursos = refRoot.getReference("cursos");
        refDias = refRoot.getReference("dias");
        refDesayunos = refRoot.getReference("desayunos");
        refComidas = refRoot.getReference("comidas");
        refAlergenos = refRoot.getReference("alergenos");
        refOfertas = refRoot.getReference("ofertas");
        refTransacciones = refRoot.getReference("transacciones");
        refTakeaway = refRoot.getReference("takeaway");
        refValoraciones = refRoot.getReference("valoraciones");


        //CargarDominio.generarBackend(getApplicationContext());

        // Listener que se activa al seleccionar una opción del menu de navegación
        nvMain.setNavigationItemSelectedListener(this);

        // Actualiza el fragmento al inicial (MainActivityFragment)
        actualizarFragmento(MainActivityFragment.class);
    }

    @Override
    public void onStart() {
        super.onStart();
        // Añadir listener de identificacion
        mAuth.addAuthStateListener(mAuthListener);
    }

    @Override
    public void onStop() {
        super.onStop();
        // Quitar listener de identificacion
        if (mAuthListener != null)
            mAuth.removeAuthStateListener(mAuthListener);
    }

    @Override
    public void onBackPressed() {
        if (dlMain.isDrawerOpen(GravityCompat.START)) {
            dlMain.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    // Al seleccionar una opción del menú de navegación, hace la acción requerida
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
            switch (item.getItemId()) {
                case R.id.nav_menu_diario:
                    actualizarFragmento(VerMenuPorDiaFragment.class);
                    break;
                case R.id.nav_ofertas:
                    actualizarFragmento(OfertasFragment.class);
                    break;
                case R.id.nav_peticiones_pendientes:
                    actualizarFragmento(PeticionesFragment.class);
                    break;
                case R.id.nav_monedero:
                    actualizarFragmento(MonederoFragment.class);
                    break;
                case R.id.nav_valorar_servicio:
                    actualizarFragmento(ValorarFragment.class);
                    break;
                case R.id.nav_info_contrato:
                    actualizarFragmento(InformacionFragment.class);
                    break;
                case R.id.nav_ajustes:
                    actualizarFragmento(AjustesFragment.class);
                    break;
                case R.id.nav_cerrar_sesion:
                    FirebaseAuth.getInstance().signOut();
                    break;
            }

        dlMain.closeDrawer(GravityCompat.START);
        return true;
    }

    // Función que actualiza el fragmento al que se le pasa por parametro
    private void actualizarFragmento(Class fragmentClass) {
        try {
            Fragment fragment = (Fragment) fragmentClass.newInstance();
            FragmentManager fragmentManager = getSupportFragmentManager();

            if (fragment != null)
                fragmentManager.beginTransaction().replace(R.id.flContent, fragment).commit();
        } catch (Exception e) {
            Snackbar.make(nvMain, R.string.error_actualizar_frgamento, Snackbar.LENGTH_LONG).show();
        }
    }

    // Inicializa el usuario y lo mantiene actualizado.
    // Tambien actualiza la UI
    private void inicializarListenerUsuario() {
        ValueEventListener usuarioListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                usuario = dataSnapshot.getValue(Usuario.class);
                tvNombreUsuario.setText(usuario.getNombre());
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        };
        refUsuarioActual.addValueEventListener(usuarioListener);
    }

    @Override
    public void onFragmentInteraction(Uri uri) {

    }

    @Override
    public void onPointerCaptureChanged(boolean hasCapture) {

    }
}
