package eus.urko.appresidencia.android.Adapter;


import android.content.Context;
import android.graphics.Color;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Paint;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import eus.urko.appresidencia.android.R;

// Adaptador de la clase alergeno para cargarla en un RecyclerView
public class AlergenoAdapter extends RecyclerView.Adapter<AlergenoAdapter.CardAlergenoHolder>{

    // Contexto de la aplicació pasada por parametro
    private Context mContext;
    // Lista con los objetos a cargar
    private List<ObjectoAuxiliar> objectoAuxiliarList;

    // Holder que hace de intermiario con la card
    public class CardAlergenoHolder extends RecyclerView.ViewHolder {
        // Interfaz UI
        private TextView tvNombre;
        private ImageView ivFoto;

        public CardAlergenoHolder(View view) {
            super(view);
            tvNombre = (TextView) view.findViewById(R.id.tvNombre);
            ivFoto = (ImageView) view.findViewById(R.id.ivFoto);
        }

        public TextView getTvNombre() {
            return tvNombre;
        }

        public void setTvNombre(TextView tvNombre) {
            this.tvNombre = tvNombre;
        }

        public ImageView getIvFoto() {
            return ivFoto;
        }

        public void setIvFoto(ImageView ivFoto) {
            this.ivFoto = ivFoto;
        }
    }

    // Constructora
    public AlergenoAdapter(Context mContext, List<ObjectoAuxiliar> objectoAuxiliarList) {
        this.mContext = mContext;
        this.objectoAuxiliarList = objectoAuxiliarList;
    }

    @Override
    public CardAlergenoHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.card_alergeno, parent, false);
        return new CardAlergenoHolder(itemView);
    }


    @Override
    public void onBindViewHolder(final CardAlergenoHolder holder, int position) {
        // Obtenemos el objeto a crear
        ObjectoAuxiliar objectoAuxiliar = objectoAuxiliarList.get(position);

        // Inicializamos la intefaz
        holder.tvNombre.setText(objectoAuxiliar.getNombre());
        holder.ivFoto.setImageResource(objectoAuxiliar.getDrawableID());

        // Dependiendo del estado del alergeno, se carga de una manera u otra
        if (objectoAuxiliar.estado) {
            // Contiene este alergeno
            holder.tvNombre.setTextColor(Color.RED);
        } else {
            // No contiene este alergeno
            // Se muestra la imagen en gris
            ColorMatrix matrix = new ColorMatrix();
            matrix.setSaturation(0);
            holder.ivFoto.setColorFilter(new ColorMatrixColorFilter(matrix));
            holder.tvNombre.setPaintFlags(holder.tvNombre.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
        }
    }

    @Override
    public int getItemCount() {
        return objectoAuxiliarList.size();
    }


    // Clase de Objeto Auxiliar que solo se utiliza en este adaptador.
    // La finalidad es hacer la implementación más sencilla.
    public static class ObjectoAuxiliar{
        private String nombre;
        private Boolean estado;
        private int drawableID;

        public ObjectoAuxiliar(String nombre, Boolean estado, int drawableID) {
            this.nombre = nombre;
            this.estado = estado;
            this.drawableID = drawableID;
        }

        public String getNombre() {
            return nombre;
        }

        public void setNombre(String nombre) {
            this.nombre = nombre;
        }

        public Boolean getEstado() {
            return estado;
        }

        public void setEstado(Boolean estado) {
            this.estado = estado;
        }

        public int getDrawableID() {
            return drawableID;
        }

        public void setDrawableID(int drawableID) {
            this.drawableID = drawableID;
        }
    }

}
