package eus.urko.appresidencia.android.Adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.ValueEventListener;

import java.util.List;

import eus.urko.appresidencia.android.Activity.MainActivity;
import eus.urko.appresidencia.android.Dialog.AlergenosDialog;
import eus.urko.appresidencia.android.Dominio.Alergeno;
import eus.urko.appresidencia.android.Dominio.ComidaAux;
import eus.urko.appresidencia.android.R;

// Adaptador de la clase comida para cargarla en un RecyclerView
public class ComidaAuxAdapter extends RecyclerView.Adapter<ComidaAuxAdapter.ComidaAuxHolder>{

    // Contexto de la aplicació pasada por parametros
    private Context mContext;
    // Lista con los objetos a cargar
    private List<ComidaAux> comidaAuxList;

    // Holder que hace de intermiario con la card
    public class ComidaAuxHolder extends RecyclerView.ViewHolder {
        // Interfaz UI
        private TextView tvNombre;
        private Button btAlergenos;

        public ComidaAuxHolder(View view) {
            super(view);
            tvNombre = (TextView) view.findViewById(R.id.tvNombre);
            btAlergenos = (Button) view.findViewById(R.id.btAlergenos);
        }

        public TextView getTvNombre() {
            return tvNombre;
        }

        public void setTvNombre(TextView tvNombre) {
            this.tvNombre = tvNombre;
        }

        public Button getBtAlergenos() {
            return btAlergenos;
        }

        public void setBtAlergenos(Button btAlergenos) {
            this.btAlergenos = btAlergenos;
        }
    }

    // Constructora
    public ComidaAuxAdapter(Context mContext, List<ComidaAux> comidaAuxList) {
        this.mContext = mContext;
        this.comidaAuxList = comidaAuxList;
    }

    @Override
    public ComidaAuxHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.card_comida_aux, parent, false);

        return new ComidaAuxHolder(itemView);
    }


    @Override
    public void onBindViewHolder(final ComidaAuxHolder holder, int position) {
        // Obtenemos el objeto a crear
        final ComidaAux comidaAux = comidaAuxList.get(position);

        // Inicializamos la intefaz
        holder.tvNombre.setText(comidaAux.getNombre());

        // Dependiendo de si exite o no el alegeno, activas o desactivamos el boton
        String idAlergenos = comidaAux.getIdAlergeno();
        if (idAlergenos == null || idAlergenos.isEmpty()) {
            holder.btAlergenos.setEnabled(false);
        } else {
            holder.btAlergenos.setEnabled(true);
            holder.btAlergenos.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    // Mostrar el dialogo del alergeno, con toda su información.
                    // Para ello, primero hay que obtenerlo
                    MainActivity.refAlergenos.child(comidaAux.getIdAlergeno()).addListenerForSingleValueEvent(new ValueEventListener() {
                        @Override
                        public void onDataChange(DataSnapshot dataSnapshot) {
                            Alergeno alergeno = dataSnapshot.getValue(Alergeno.class);
                            if (alergeno != null) {
                                AlergenosDialog alergenosDialog = new AlergenosDialog(mContext, alergeno);
                                alergenosDialog.show();
                            }
                        }

                        @Override
                        public void onCancelled(DatabaseError databaseError) {
                        }
                    });
                }
            });
        }


    }

    @Override
    public int getItemCount() {
        return comidaAuxList.size();
    }


}
