package eus.urko.appresidencia.android.Adapter;

import android.content.Context;
import android.content.DialogInterface;
import android.support.annotation.NonNull;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.util.List;

import eus.urko.appresidencia.android.Activity.MainActivity;
import eus.urko.appresidencia.android.Dominio.Oferta;
import eus.urko.appresidencia.android.Dominio.Takeaway;
import eus.urko.appresidencia.android.R;

// Adaptador de la clase peticiones para cargarla en un RecyclerView
public class PeticionesAdapter extends RecyclerView.Adapter<PeticionesAdapter.PeticionesHolder>{

    // Contexto de la aplicació pasada por parametro
    private Context mContext;
    // Lista con los objetos a cargar
    private List<Takeaway> takeawayList;
    private List<String> idTakeawayList;

    // Holder que hace de intermiario con la card
    public class PeticionesHolder extends RecyclerView.ViewHolder {
        // Interfaz UI
        private TextView tvTitulo;
        private Button btCancelar;

        public PeticionesHolder(View view) {
            super(view);
            tvTitulo = (TextView) view.findViewById(R.id.tvTitulo);
            btCancelar = (Button) view.findViewById(R.id.btCancelar);
        }

        public TextView getTvTitulo() {
            return tvTitulo;
        }

        public void setTvTitulo(TextView tvTitulo) {
            this.tvTitulo = tvTitulo;
        }

        public Button getBtCancelar() {
            return btCancelar;
        }

        public void setBtCancelar(Button btCancelar) {
            this.btCancelar = btCancelar;
        }
    }

    // Constructora
    public PeticionesAdapter(Context mContext, List<Takeaway> takeawayList, List<String> idTakeawayList) {
        this.mContext = mContext;
        this.takeawayList = takeawayList;
        this.idTakeawayList = idTakeawayList;
    }

    @Override
    public PeticionesHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.card_peticiones, parent, false);

        return new PeticionesHolder(itemView);
    }


    @Override
    public void onBindViewHolder(final PeticionesHolder holder, final int position) {
        // Obtenemos el objeto a crear
        final Takeaway takeaway = takeawayList.get(position);

        // Inicializamos la intefaz
        holder.tvTitulo.setText(mContext.getString(R.string.texto_peticion_1) + " " + takeaway.getTipoComida() + " " + mContext.getString(R.string.texto_peticion_2) + " " + takeaway.getFechaComida());

        // Listener para el boton del holder
        holder.btCancelar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                // Crear el builder del dialogo
                final AlertDialog.Builder builder = new AlertDialog.Builder(mContext);

                // Añdir el título y el mensaje
                builder.setMessage(R.string.pregunta_cancelar_peticion)
                        .setTitle(R.string.cancelar_takeaway);

                // Boton negativo
                builder.setPositiveButton(R.string.cancelar, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        //Cancela la peticion
                        MainActivity.refTakeaway
                                .child(idTakeawayList.get(position))
                                .removeValue(new DatabaseReference.CompletionListener() {
                            @Override
                            public void onComplete(DatabaseError databaseError, DatabaseReference databaseReference) {
                                Toast.makeText(mContext, R.string.aviso_cancelacion, Toast.LENGTH_SHORT).show();
                            }
                        });

                        // Cerrar dialogo
                        dialogInterface.dismiss();
                    }
                });

                builder.setNegativeButton(R.string.cerrar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogInterface, int i) {
                        // Cerrar dialogo
                        dialogInterface.dismiss();
                    }
                });

                // Crear el dialogo
                AlertDialog alertDialog = builder.create();

                // Mostrar el dialogo
                alertDialog.show();
            }
        });
    }

    @Override
    public int getItemCount() {
        return takeawayList.size();
    }


}
