package eus.urko.appresidencia.android.Fragments;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;

import eus.urko.appresidencia.android.Dominio.Curso;
import eus.urko.appresidencia.android.Dominio.Residencia;
import eus.urko.appresidencia.android.Activity.MainActivity;
import eus.urko.appresidencia.android.R;

public class AjustesFragment extends Fragment {
    private OnFragmentInteractionListener mListener;
    // TAG para identificar el fragment
    private static final String TAG = "AjustesFragment";

    // Referencias UI
    private View viewFragment;
    private Spinner spCurso;
    private Spinner spResidencia;
    private Button btSeleccionar;

    // Adaptadores y arrays de las Residencias
    private List<String> listaResidencias;
    private List<String> listaCursos;
    private List<String> listaResidenciasID;
    private List<String> listaCursosID;

    public AjustesFragment() {
        // Required empty public constructor
    }

    public static AjustesFragment newInstance(String param1, String param2) {
        AjustesFragment fragment = new AjustesFragment();

        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        poblarSpinnerResidencia();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Cargar referencias UI
        viewFragment = inflater.inflate(R.layout.fragment_ajustes, container, false);
        btSeleccionar = (Button) viewFragment.findViewById(R.id.btSeleccionar);
        spResidencia = (Spinner) viewFragment.findViewById(R.id.spResidencia);
        spCurso = (Spinner) viewFragment.findViewById(R.id.spCurso);

        // Inializar el boton de btSeleccionar
        btSeleccionar.setEnabled(false);
        // Listener del boton btSeleccionar
        btSeleccionar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                MainActivity.usuario.setIdResidenciaPreferida(listaResidenciasID.get(spResidencia.getSelectedItemPosition()));
                MainActivity.usuario.setIdCursoPreferido(listaCursosID.get(spCurso.getSelectedItemPosition()));
                MainActivity.refUsuarioActual.setValue(MainActivity.usuario);

            }
        });

        // Al selecionar un elemento del spinner de la residencia puebla el del curso, sino lo borra.
        spResidencia.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                poblarSpinnerCurso();
                spCurso.setEnabled(true);
                btSeleccionar.setEnabled(false);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                spCurso.setEnabled(false);
                btSeleccionar.setEnabled(false);
            }
        });

        // Desactiva el spinner del curso
        spCurso.setEnabled(false);

        // Al selecionar un elemento del spinner del curso, activa el boton de guardar
        spCurso.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                btSeleccionar.setEnabled(true);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                btSeleccionar.setEnabled(false);
            }
        });

        return viewFragment;
    }

    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(Uri uri);
    }

    // Pruebla el spinner con las residencias
    private void poblarSpinnerResidencia () {
        MainActivity.refResindecias.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                listaResidencias = new ArrayList<String>();
                listaResidenciasID = new ArrayList<String>();

                for (DataSnapshot resSnapshot: dataSnapshot.getChildren()) {
                    Residencia res = resSnapshot.getValue(Residencia.class);
                    listaResidencias.add(res.getNombre());
                    listaResidenciasID.add(resSnapshot.getKey());
                }

                ArrayAdapter<String> adp1 = new ArrayAdapter<String>(getActivity(),
                        android.R.layout.simple_list_item_1, listaResidencias);
                adp1.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                spResidencia.setAdapter(adp1);
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                // Getting Post failed, log a message
                Log.w(TAG, "loadPost:onCancelled", databaseError.toException());
                // ...
            }
        });
    }

    // Puebla el spinner con los curso de la residencia
    private void poblarSpinnerCurso () {
        String identificadorRes = listaResidenciasID.get(spResidencia.getSelectedItemPosition());

        Query query = MainActivity.refCursos.
                orderByChild("idResidencia").
                equalTo(identificadorRes);

        query.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                listaCursos = new ArrayList<String>();
                listaCursosID = new ArrayList<String>();

                for (DataSnapshot resSnapshot: dataSnapshot.getChildren()) {
                    Curso res = resSnapshot.getValue(Curso.class);
                    listaCursos.add(res.getNombre());
                    listaCursosID.add(resSnapshot.getKey());
                }

                ArrayAdapter<String> adp1 = new ArrayAdapter<String>(getActivity(),
                        android.R.layout.simple_list_item_1, listaCursos);
                adp1.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                spCurso.setAdapter(adp1);

            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                // Getting Post failed, log a message
                Log.w(TAG, "loadPost:onCancelled", databaseError.toException());
                // ...
            }
        });
    }

}
