package eus.urko.appresidencia.android.Fragments;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.database.Transaction;

import eus.urko.appresidencia.android.Activity.MainActivity;
import eus.urko.appresidencia.android.Dominio.Transaccion;
import eus.urko.appresidencia.android.R;


public class MonederoFragment extends Fragment {
    private OnFragmentInteractionListener mListener;
    // TAG para identificar el fragment
    private static final String TAG = "MonederoFragment";

    // Referencias UI
    private View viewFragment;
    private TextView tvSaldoActual;
    private EditText etNombreTarjeta;
    private EditText etNumeroTarjeta;
    private EditText etFechaTarjeta;
    private EditText etCSVTarjeta;
    private EditText etImporteRecarga;
    private Button btRecargar;

    // Infomacion para la interfaz de la tarjeta de la tarjeta
    private static final int CARD_NUMBER_TOTAL_SYMBOLS = 19; // size of pattern 0000-0000-0000-0000
    private static final int CARD_NUMBER_TOTAL_DIGITS = 16; // max numbers of digits in pattern: 0000 x 4
    private static final int CARD_NUMBER_DIVIDER_MODULO = 5; // means divider position is every 5th symbol beginning with 1
    private static final int CARD_NUMBER_DIVIDER_POSITION = CARD_NUMBER_DIVIDER_MODULO - 1; // means divider position is every 4th symbol beginning with 0
    private static final char CARD_NUMBER_DIVIDER = '-';
    private static final int CARD_DATE_TOTAL_SYMBOLS = 5; // size of pattern MM/YY
    private static final int CARD_DATE_TOTAL_DIGITS = 4; // max numbers of digits in pattern: MM + YY
    private static final int CARD_DATE_DIVIDER_MODULO = 3; // means divider position is every 3rd symbol beginning with 1
    private static final int CARD_DATE_DIVIDER_POSITION = CARD_DATE_DIVIDER_MODULO - 1; // means divider position is every 2nd symbol beginning with 0
    private static final char CARD_DATE_DIVIDER = '/';
    private static final int CARD_CVC_TOTAL_SYMBOLS = 3;

    public MonederoFragment() {
        // Required empty public constructor
    }

    public static MonederoFragment newInstance(String param1, String param2) {
        MonederoFragment fragment = new MonederoFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        // Cargar referencias UI
        viewFragment = inflater.inflate(R.layout.fragment_monedero, container, false);
        tvSaldoActual = (TextView) viewFragment.findViewById(R.id.tvSaldoActual);
        etNombreTarjeta = (EditText) viewFragment.findViewById(R.id.etNombreTarjeta);
        etNumeroTarjeta = (EditText) viewFragment.findViewById(R.id.etNumeroTarjeta);
        etFechaTarjeta = (EditText) viewFragment.findViewById(R.id.etFechaTarjeta);
        etCSVTarjeta = (EditText) viewFragment.findViewById(R.id.etCSVTarjeta);
        etImporteRecarga = (EditText) viewFragment.findViewById(R.id.etImporteRecarga);
        btRecargar = (Button) viewFragment.findViewById(R.id.btRecargar);

        // Cargar saldo actual
        actualizarSaldoActual();

        // Hacer que la interfaz del número de la tarjeta sea "bonita"
        etNumeroTarjeta.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (!isInputCorrect(editable, CARD_NUMBER_TOTAL_SYMBOLS, CARD_NUMBER_DIVIDER_MODULO, CARD_NUMBER_DIVIDER)) {
                    editable.replace(0, editable.length(), concatString(getDigitArray(editable, CARD_NUMBER_TOTAL_DIGITS), CARD_NUMBER_DIVIDER_POSITION, CARD_NUMBER_DIVIDER));
                }

            }
        });

        // Hacer que la interfaz de la fecha de la tarjeta sea "bonita"
        etFechaTarjeta.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (!isInputCorrect(editable, CARD_DATE_TOTAL_SYMBOLS, CARD_DATE_DIVIDER_MODULO, CARD_DATE_DIVIDER)) {
                    editable.replace(0, editable.length(), concatString(getDigitArray(editable, CARD_DATE_TOTAL_DIGITS), CARD_DATE_DIVIDER_POSITION, CARD_DATE_DIVIDER));
                }

            }
        });

        // Hacer que la interfaz del CSV de la tarjeta sea "bonita"
        etCSVTarjeta.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (editable.length() > CARD_CVC_TOTAL_SYMBOLS) {
                    editable.delete(CARD_CVC_TOTAL_SYMBOLS, editable.length());
                }

            }
        });

        // Listener del boton btRecargar
        btRecargar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                intentarRecarga();
            }
        });

        return viewFragment;
    }

    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(Uri uri);
    }

    // Actualiza el saldo de la interfaz
    private void actualizarSaldoActual() {
        tvSaldoActual.setText(getString(R.string.saldo_Actual) + MainActivity.usuario.getSaldo() + "€");
    }

    private void intentarRecarga() {
        // Quitar errores
        etNombreTarjeta.setError(null);
        etNumeroTarjeta.setError(null);
        etFechaTarjeta.setError(null);
        etCSVTarjeta.setError(null);
        etImporteRecarga.setError(null);


        // Obtener los datos introducidos
        String nombreTarjeta = etNombreTarjeta.getText().toString(); // Obtener el nombre
        String numeroTarjeta = etNumeroTarjeta.getText().toString(); // Obtener el numero
        String fechaTarjeta = etFechaTarjeta.getText().toString(); // Obtener la fecha
        String cSVTarjeta = etCSVTarjeta.getText().toString(); // Obtener el CSV
        final int importeRecarga;
        if (etImporteRecarga.getText().toString().isEmpty()) importeRecarga = 0;
        else importeRecarga = Integer.valueOf(etImporteRecarga.getText().toString()); // Obtener el importe
        boolean cancelar = false; // Permite saber si hay que cancelar el intento de recarga
        View focusView = null; // View donde hacer el focus en caso de error

        // Comprobar que el importe es valido
        if (importeRecarga < 5 || importeRecarga > 100) {
            etImporteRecarga.setError(getString(R.string.informacion_recarga));
            focusView = etImporteRecarga;
            cancelar = true;
        }

        // Comprobar que CSV es correcto
        if (cSVTarjeta.length() != 3) {
            etCSVTarjeta.setError(getString(R.string.csv_no_valido));
            focusView = etCSVTarjeta;
            cancelar = true;
        }

        // Comprobar que la fecha de la tarjeta es correcta
        if (fechaTarjeta.length() != 5) {
            etFechaTarjeta.setError(getString(R.string.fecha_tarjeta_no_valido));
            focusView = etFechaTarjeta;
            cancelar = true;
        }

        // Comprobar que el número de tarjeta es correcto
        if (numeroTarjeta.length() != 19) {
            etNumeroTarjeta.setError(getString(R.string.numero_tarjeta_no_valido));
            focusView = etNumeroTarjeta;
            cancelar = true;
        }

        // Comprobar que el nombre de la tarjeta es correcto
        if (nombreTarjeta.length() < 10 && nombreTarjeta.length() > 100) {
            etNombreTarjeta.setError(getString(R.string.nombre_tarjeta__no_valido));
            focusView = etNombreTarjeta;
            cancelar = true;
        }

        if (cancelar)
            // Ha habido un error, el proceso se para
            // Se hace focus en el error
            focusView.requestFocus();
        else {
            // Añadir la transaccion y actualizar el saldo
            Transaccion transaccion = new Transaccion(importeRecarga, getString(R.string.recarga), MainActivity.firebaseUser.getUid());
            String keyTransaccion = MainActivity.refTransacciones.push().getKey();
            MainActivity.refTransacciones.child(keyTransaccion).setValue(transaccion).addOnCompleteListener(new OnCompleteListener<Void>() {
                @Override
                public void onComplete(@NonNull Task<Void> task) {
                    MainActivity.refUsuarioActual.child("saldo").setValue(MainActivity.usuario.getSaldo() + importeRecarga).addOnCompleteListener(new OnCompleteListener<Void>() {
                        @Override
                        public void onComplete(@NonNull Task<Void> task) {
                            actualizarSaldoActual();
                            btRecargar.setEnabled(true);
                        }
                    });
                }
            });

            // Restablecer interfaz
            etNombreTarjeta.setText("");
            etNumeroTarjeta.setText("");
            etFechaTarjeta.setText("");
            etCSVTarjeta.setText("");
            etImporteRecarga.setText("");
            btRecargar.setEnabled(false);
        }
    }

    // Metodos que comprueban la información de la tarjeta

    private boolean isInputCorrect(Editable s, int size, int dividerPosition, char divider) {
        boolean isCorrect = s.length() <= size;
        for (int i = 0; i < s.length(); i++) {
            if (i > 0 && (i + 1) % dividerPosition == 0) {
                isCorrect &= divider == s.charAt(i);
            } else {
                isCorrect &= Character.isDigit(s.charAt(i));
            }
        }
        return isCorrect;
    }

    private String concatString(char[] digits, int dividerPosition, char divider) {
        final StringBuilder formatted = new StringBuilder();

        for (int i = 0; i < digits.length; i++) {
            if (digits[i] != 0) {
                formatted.append(digits[i]);
                if ((i > 0) && (i < (digits.length - 1)) && (((i + 1) % dividerPosition) == 0)) {
                    formatted.append(divider);
                }
            }
        }

        return formatted.toString();
    }

    private char[] getDigitArray(final Editable s, final int size) {
        char[] digits = new char[size];
        int index = 0;
        for (int i = 0; i < s.length() && index < size; i++) {
            char current = s.charAt(i);
            if (Character.isDigit(current)) {
                digits[index] = current;
                index++;
            }
        }
        return digits;
    }
}

