package eus.urko.appresidencia.android.Fragments;

import android.app.DatePickerDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.Query;
import com.google.firebase.database.ServerValue;
import com.google.firebase.database.ValueEventListener;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import eus.urko.appresidencia.android.Dominio.Comida;
import eus.urko.appresidencia.android.Dominio.ComidaAux;
import eus.urko.appresidencia.android.Adapter.ComidaAuxAdapter;
import eus.urko.appresidencia.android.Dominio.Desayuno;
import eus.urko.appresidencia.android.Dominio.Dia;
import eus.urko.appresidencia.android.Activity.MainActivity;
import eus.urko.appresidencia.android.Dominio.Takeaway;
import eus.urko.appresidencia.android.R;

public class VerMenuPorDiaFragment extends Fragment {
    private OnFragmentInteractionListener mListener;
    // TAG para identificar el fragment
    private static final String TAG = "VerMenuPorDiaFragment";

    // Referencias UI
    private View viewFragment;
    private Button btCambiarFecha;
    private Button btUnDiaMas;
    private Button btEncargarParaLlevar;
    private TextView tvFecha;
    private TabHost tabHost;
    private TabHost.TabSpec tbDesayuno;
    private TabHost.TabSpec tbComida;
    private TabHost.TabSpec tbCena;
    private RecyclerView rvDesayuno;
    private RecyclerView rvComida;
    private RecyclerView rvCena;

    // Objetos que identifican la fecha
    private Calendar fechaEscogida = Calendar.getInstance();;
    private SimpleDateFormat dateFormatterConBarras = new SimpleDateFormat("dd/MM/yyyy");
    private SimpleDateFormat dateFormatterSinBarras = new SimpleDateFormat("yyyyMMdd");

    // Objeto del día escogido
    private Dia diaEscogido;
    private String idDiaEscogido;
    private Desayuno desayunoEscogido;
    private String idDesayunoEscogido = "";
    private Comida comidaEscogida;
    private String idComidaEscogido = "";
    private Comida cenaEscogida;
    private String idCenaEscogido = "";

    // Adaptadores y arrays de las comidas
    private ComidaAuxAdapter desayunoAdapter;
    private ComidaAuxAdapter comidaAdapter;
    private ComidaAuxAdapter cenaAdapter;
    private List<ComidaAux> desayunoAuxList;
    private List<ComidaAux> comidaAuxList;
    private List<ComidaAux> cenaAuxList;

    public VerMenuPorDiaFragment() {
        // Required empty public constructor
    }

    public static VerMenuPorDiaFragment newInstance(String param1, String param2) {
        VerMenuPorDiaFragment fragment = new VerMenuPorDiaFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Cargar referencias UI
        viewFragment = inflater.inflate(R.layout.fragment_ver_menu_por_dia, container, false);
        btCambiarFecha = (Button) viewFragment.findViewById(R.id.btCambiarFecha);
        btUnDiaMas = (Button) viewFragment.findViewById(R.id.btUnDiaMas);
        tvFecha = (TextView) viewFragment.findViewById(R.id.tvFecha);
        tabHost = (TabHost) viewFragment.findViewById(R.id.tabHost);
        rvDesayuno = (RecyclerView) viewFragment.findViewById(R.id.rvDesayuno);
        rvComida = (RecyclerView) viewFragment.findViewById(R.id.rvComida);
        rvCena = (RecyclerView) viewFragment.findViewById(R.id.rvCena);
        btEncargarParaLlevar = (Button) viewFragment.findViewById(R.id.btEncargarParaLlevar);

        // Incializar las pestañas
        tabHost.setup();

        // Pestaña desayuno
        tbDesayuno = tabHost.newTabSpec("tbDesayuno");
        tbDesayuno.setIndicator(getString(R.string.desayuno));
        tbDesayuno.setContent(R.id.tbDesayuno);
        tabHost.addTab(tbDesayuno);

        // Pestaña comida
        tbComida = tabHost.newTabSpec("tbComida");
        tbComida.setIndicator(getString(R.string.comida));
        tbComida.setContent(R.id.tbComida);
        tabHost.addTab(tbComida);

        // Pestaña cena
        tbCena = tabHost.newTabSpec("tbCena");
        tbCena.setIndicator(getString(R.string.cena));
        tbCena.setContent(R.id.tbCena);
        tabHost.addTab(tbCena);

        // Listener boton btCambiarFecha
        btCambiarFecha.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                dialogoCambiarFecha();
            }
        });

        // Listener boton btUnDiaMas
        btUnDiaMas.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                // Añade un día a la fecha escogida
                fechaEscogida.add(Calendar.DATE, 1);
                actualizarUI();
            }
        });

        // Listener boton btEncargarParaLlevar
        btEncargarParaLlevar.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                intentarPedirComidaParaLlevar();
            }
        });

        // Cargar el día de hoy
        fechaEscogida.setTime(new Date());

        // Actualizar UI por primera vez
        actualizarUI();

        // Inicializar adaptadores de las pestañas
        desayunoAuxList = new ArrayList<>();
        comidaAuxList = new ArrayList<>();
        cenaAuxList = new ArrayList<>();

        desayunoAdapter = new ComidaAuxAdapter(getContext(), desayunoAuxList);
        GridLayoutManager mLayoutManagerDesayuno = new GridLayoutManager(getContext(), 1);
        rvDesayuno.setLayoutManager(mLayoutManagerDesayuno);
        rvDesayuno.setItemAnimator(new DefaultItemAnimator());
        rvDesayuno.setAdapter(desayunoAdapter);
        
        comidaAdapter = new ComidaAuxAdapter(getContext(), comidaAuxList);
        GridLayoutManager mLayoutManagerComida = new GridLayoutManager(getContext(), 2);
        mLayoutManagerComida.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                switch (position) {
                    case 0 :
                    case 5 :
                        return 2;
                    default:
                        return 1;
                }
            }
        });
        rvComida.setLayoutManager(mLayoutManagerComida);
        rvComida.setItemAnimator(new DefaultItemAnimator());
        rvComida.setAdapter(comidaAdapter);

        cenaAdapter = new ComidaAuxAdapter(getContext(), cenaAuxList);
        GridLayoutManager mLayoutManagerCena = new GridLayoutManager(getContext(), 2);
        mLayoutManagerCena.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                switch (position) {
                    case 0 :
                    case 5 :
                        return 2;
                    default:
                        return 1;
                }
            }
        });
        rvCena.setLayoutManager(mLayoutManagerCena);
        rvCena.setItemAnimator(new DefaultItemAnimator());
        rvCena.setAdapter(cenaAdapter);

        return viewFragment;
    }



    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }


    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(Uri uri);
    }

    // Abre un dialogo para cambiar la fecha
    private void dialogoCambiarFecha(){
        DatePickerDialog dialog = new DatePickerDialog(getContext(),
                new DatePickerDialog.OnDateSetListener() {
            @Override
            public void onDateSet(DatePicker datePicker, int i, int i1, int i2) {
                fechaEscogida.set(Calendar.YEAR, datePicker.getYear());
                fechaEscogida.set(Calendar.MONTH, datePicker.getMonth());
                fechaEscogida.set(Calendar.DAY_OF_MONTH, datePicker.getDayOfMonth());

                actualizarUI();
            }
        },
                fechaEscogida.get(Calendar.YEAR),
                fechaEscogida.get(Calendar.MONTH),
                fechaEscogida.get(Calendar.DAY_OF_MONTH));
        dialog.show();
    }

    // Actualiza la UI con las comidas del dia en particulas
    private void actualizarUI(){
        tvFecha.setText(getString(R.string.fecha) + "  " + dateFormatterConBarras.format(fechaEscogida.getTime()) + "   ");

        String fecha = dateFormatterSinBarras.format(fechaEscogida.getTime());

        Query query = MainActivity.refDias.
                orderByChild("fechaidCursoidResidencia").
                equalTo(fecha + MainActivity.usuario.getIdCursoPreferido() + MainActivity.usuario.getIdResidenciaPreferida());

        // Al obtener el dia, si este existe carga en cada pestaña la informacion correspondiente.
        query.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    DataSnapshot dataSnapshot1 = dataSnapshot.getChildren().iterator().next();
                    idDiaEscogido = dataSnapshot.getKey();
                    diaEscogido = dataSnapshot1.getValue(Dia.class);

                    MainActivity.refDesayunos.child(diaEscogido.getIdDesayuno()).addListenerForSingleValueEvent(new ValueEventListener() {
                        @Override
                        public void onDataChange(DataSnapshot dataSnapshot) {
                            idDesayunoEscogido = dataSnapshot.getKey();
                            desayunoEscogido = dataSnapshot.getValue(Desayuno.class);
                            desayunoAuxList.clear();
                            if (desayunoEscogido != null) {
                                desayunoAuxList.add(desayunoEscogido.getBuffet());
                            }
                            desayunoAdapter.notifyDataSetChanged();
                        }

                        @Override
                        public void onCancelled(DatabaseError databaseError) {
                            desayunoEscogido = null;
                            idDesayunoEscogido = "";
                            desayunoAuxList.clear();
                            desayunoAdapter.notifyDataSetChanged();
                        }
                    });

                    MainActivity.refComidas.child(diaEscogido.getIdComida()).addListenerForSingleValueEvent(new ValueEventListener() {
                        @Override
                        public void onDataChange(DataSnapshot dataSnapshot) {
                            idComidaEscogido = dataSnapshot.getKey();
                            comidaEscogida = dataSnapshot.getValue(Comida.class);
                            comidaAuxList.clear();
                            if (comidaEscogida != null) {
                                comidaAuxList.add(comidaEscogida.getBuffet());
                                comidaAuxList.add(comidaEscogida.getPrimero1());
                                comidaAuxList.add(comidaEscogida.getPrimero2());
                                comidaAuxList.add(comidaEscogida.getSegundo1());
                                comidaAuxList.add(comidaEscogida.getSegundo2());
                                comidaAuxList.add(comidaEscogida.getPostre());
                            }
                            comidaAdapter.notifyDataSetChanged();
                        }

                        @Override
                        public void onCancelled(DatabaseError databaseError) {
                            idComidaEscogido = "";
                            comidaEscogida = null;
                            comidaAuxList.clear();
                            cenaAdapter.notifyDataSetChanged();
                        }
                    });

                    MainActivity.refComidas.child(diaEscogido.getIdCena()).addListenerForSingleValueEvent(new ValueEventListener() {
                        @Override
                        public void onDataChange(DataSnapshot dataSnapshot) {
                            idCenaEscogido = dataSnapshot.getKey();
                            cenaEscogida = dataSnapshot.getValue(Comida.class);
                            cenaAuxList.clear();
                            if (cenaEscogida != null) {
                                cenaAuxList.add(cenaEscogida.getBuffet());
                                cenaAuxList.add(cenaEscogida.getPrimero1());
                                cenaAuxList.add(cenaEscogida.getPrimero2());
                                cenaAuxList.add(cenaEscogida.getSegundo1());
                                cenaAuxList.add(cenaEscogida.getSegundo2());
                                cenaAuxList.add(cenaEscogida.getPostre());

                            }
                            cenaAdapter.notifyDataSetChanged();
                        }

                        @Override
                        public void onCancelled(DatabaseError databaseError) {
                            idCenaEscogido = "";
                            cenaEscogida = null;
                            cenaAuxList.clear();
                            cenaAdapter.notifyDataSetChanged();
                        }
                    });
                } else {
                    diaEscogido = null;
                    desayunoEscogido = null;
                    desayunoAuxList.clear();
                    desayunoAdapter.notifyDataSetChanged();
                    comidaEscogida = null;
                    comidaAuxList.clear();
                    cenaAdapter.notifyDataSetChanged();
                    cenaEscogida = null;
                    cenaAuxList.clear();
                    cenaAdapter.notifyDataSetChanged();

                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {

            }
        });
    }

    // Inteta pedir la comida seleccionada para llevar.
    public void intentarPedirComidaParaLlevar(){
        // Componentes UI de dentro del dialogo
        final EditText etComentarios = new EditText(getContext());
        final FrameLayout flDialogo = new FrameLayout(getContext());

        // Crear layourParams con los margenes
        FrameLayout.LayoutParams layoutParams = new  FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        layoutParams.leftMargin = getResources().getDimensionPixelSize(R.dimen.activity_horizontal_margin);
        layoutParams.rightMargin = getResources().getDimensionPixelSize(R.dimen.activity_horizontal_margin);

        // Añadir los magernees etComentarios
        etComentarios.setLayoutParams(layoutParams);

        // Poner el hint del etComentarios
        etComentarios.setHint(R.string.comentario);

        // Añadir el editText a flDialogo
        flDialogo.addView(etComentarios);

        // Crear el builder del dialogo
        final AlertDialog.Builder builder = new AlertDialog.Builder(getContext());

        // Añdir el título y el mensaje
        builder.setMessage(R.string.pregunta_encargar_takeaway)
                .setTitle(R.string.takeaway);

        // Boton negativo
        builder.setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                // Cerrar dialogo
                dialogInterface.dismiss();
            }
        });

        builder.setPositiveButton(R.string.aceptar_dialogo, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialogInterface, int i) {
                // Borra la peticion de comida para llevar.
                String idComida = "";
                String tipoComida = "";
                switch (tabHost.getCurrentTab()) {
                    case 0:
                        idComida = idDesayunoEscogido;
                        tipoComida = getString(R.string.desayuno);
                        break;
                    case 1:
                        idComida = idComidaEscogido;
                        tipoComida = getString(R.string.comida);
                        break;
                    case 2:
                        idComida = idCenaEscogido;
                        tipoComida = getString(R.string.cena);
                        break;
                }

                Takeaway takeaway = new Takeaway(new Date().toString(),
                        dateFormatterConBarras.format(fechaEscogida.getTime()),
                        tipoComida,
                        MainActivity.firebaseUser.getUid(),
                        idComida,
                        idDiaEscogido,
                        MainActivity.usuario.getIdCursoPreferido(),
                        MainActivity.usuario.getIdResidenciaPreferida());

                String keyTakeAway = MainActivity.refTakeaway.push().getKey();
                MainActivity.refTakeaway.child(keyTakeAway).setValue(takeaway).addOnCompleteListener(new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        Toast.makeText(getContext(), R.string.takeaway_ok, Toast.LENGTH_SHORT).show();
                    }
                });
                dialogInterface.dismiss();
            }
        });

        // Crear el dialogo
        AlertDialog alertDialog = builder.create();

        // Asignar el view al dialogo
        alertDialog.setView(flDialogo);

        // Mostrar el dialogo
        alertDialog.show();
    }


}
